#include "Event.hh"
#include <ace/OS.h>
#include <tao/corba.h>

#include <Common/Accessor.hh>

#include <iostream.h>
#include <stdlib.h>

#define LOG_EVENTS 10

#define NULL_OBJECT_REFERENCE -1
#define LOG_FULL -2
#define LOG_LOCKED -3
#define INVALID_GRAMMAR -4
#define INVALID_CONSTRAINT -5
#define SYSTEM_EXCEPTION -6
#define INVALID_LOG_FULL_ACTION -7
#define INVALID_THRESHOLD -8
#define ALREADY_CONNECTED -9
#define DISCONNECTED -10

Event::Event (CORBA::ORB_var orb) : POA_CosEventComm::PushSupplier ()
{
   this->orb = orb;
}


DsEventLogAdmin::EventLogFactory_var Event::resolve (CORBA::ORB_var orb)
{
   CORBA::Object_var object = NULL;

   if (CORBA::is_nil(orb.in ()))
   {
      cout << "ERROR: ORB not initialised." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   try
   {
      object = orb->resolve_initial_references ("EventLogFactory");
   }
   catch(...)
   {
      cerr << "Failed to resolve EventLogFactory..." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   DsEventLogAdmin::EventLogFactory_var factory = DsEventLogAdmin::EventLogFactory::_narrow (object.in ());

   if (CORBA::is_nil(factory.in ()))
   {
      cout << "ERROR: Cannot get Basic Log factory" << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   this->event_log_factory = factory;

   cout << "Resolved Event Log Factory" << endl;

   return factory;
}

DsEventLogAdmin::EventLog_var Event::create (DsEventLogAdmin::EventLogFactory_var factory)
{
   if (CORBA::is_nil(factory.in ()))
   {
      cout << "ERROR: Cannot get Event Log factory" << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   DsLogAdmin::LogFullActionType logFullAction = DsLogAdmin::wrap;
   CORBA::ULongLong max_size = 0; // 0 means "infinite"
   DsLogAdmin::LogId logId = 0;
   DsLogAdmin::CapacityAlarmThresholdList_var thres = new DsLogAdmin::CapacityAlarmThresholdList ((CORBA::ULong)75);
   DsEventLogAdmin::EventLog_var event_log;

   try
   {
      event_log = factory->create (logFullAction, max_size, thres, logId);
   }
   catch(const DsLogAdmin::InvalidLogFullAction&)
   {
      cerr << "Event Log creation failed with InvalidLogFullAction!" << endl;
      exit (INVALID_LOG_FULL_ACTION);
   }
   catch(const DsLogAdmin::InvalidThreshold&)
   {
      cerr << "Event Log creation failed with InvalidThreshold!" << endl;
      exit (INVALID_THRESHOLD);
   }

   if (CORBA::is_nil(event_log.in ()))
   {
      cout << "ERROR: Event Log is null." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   cout << "Created log with id :" << logId << endl;

   return event_log;
}

void Event::connect (DsEventLogAdmin::EventLog_var channel)
{
   CosEventChannelAdmin::SupplierAdmin_var admin = channel->for_suppliers ();
   CosEventComm::PushSupplier_var supplier;

   try
   {
      proxy = admin->obtain_push_consumer ();
      supplier = CosEventComm::PushSupplier::_narrow (getCORBARef ().in ());

      proxy->connect_push_supplier (supplier.in ());
   }
   catch(const CosEventChannelAdmin::AlreadyConnected&)
   {
      cerr << "Already connected!"<< endl;
      exit (ALREADY_CONNECTED);
   }

   cout << "Connected supplier to log channel!" << endl;
}

void Event::generate ()
{
   CORBA::Any any;
   const char* message;

   for (int i=0; i < LOG_EVENTS; i++)
   {
      message = "Event Log message #";

      any <<= ACE_OS::strdup (message);

      try
      {
         proxy->push (any);
      }
      catch(...)
      {
         cerr << "Proxy is disconnected..."<< endl;
         exit (DISCONNECTED);
      }
   }

   cout <<"Generated some log messages..." << endl;
}

void Event::disconnect ()
{
   try
   {
      proxy->disconnect_push_consumer ();
   }
   catch(const CORBA::SystemException& ex)
   {
      cerr << "Caught " << ex << " while disconnecting." << endl;
      exit (SYSTEM_EXCEPTION);
   }
   catch(...)
   {
      cerr << "Unexpected exception" << endl;
      exit (SYSTEM_EXCEPTION);
   }

   cout << "Disconnected supplier from log channel." << endl;
}
