#include <ace/OS.h>
#include "Notify.hh"
#include <tao/corba.h>

#include <Common/Accessor.hh>

#include <iostream.h>
#include <stdlib.h>

#define LOG_EVENTS 4

#define NULL_OBJECT_REFERENCE -1
#define LOG_FULL -2
#define LOG_LOCKED -3
#define INVALID_GRAMMAR -4
#define INVALID_CONSTRAINT -5
#define SYSTEM_EXCEPTION -6
#define INVALID_LOG_FULL_ACTION -7
#define INVALID_THRESHOLD -8
#define ALREADY_CONNECTED -9
#define DISCONNECTED -10
#define UNSUPPORTED_QOS -11
#define UNSUPPORTED_ADMIN -12
#define ADMIN_LIMIT_EXCEEDED -13

Notify::Notify (CORBA::ORB_var orb) : POA_CosNotifyComm::PushSupplier ()
{
   this->orb = orb;
}


DsNotifyLogAdmin::NotifyLogFactory_var Notify::resolve (CORBA::ORB_var orb)
{
   CORBA::Object_var object = NULL;

   if (CORBA::is_nil(orb.in ()))
   {
      cout << "ERROR: ORB not initialised." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   try
   {
      object = orb->resolve_initial_references ("NotifyLogFactory");
   }
   catch(...)
   {
      cerr << "Failed to resolve NotifyLogFactory..." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   DsNotifyLogAdmin::NotifyLogFactory_var factory = DsNotifyLogAdmin::NotifyLogFactory::_narrow (object.in ());

   if (CORBA::is_nil(factory.in ()))
   {
      cout << "ERROR: Cannot get Notify Log factory" << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   this->notify_log_factory = factory;

   cout << "Resolved Notify Log Factory" << endl;

   return factory;
}

DsNotifyLogAdmin::NotifyLog_var Notify::create (DsNotifyLogAdmin::NotifyLogFactory_var factory)
{
   if (CORBA::is_nil(factory.in ()))
   {
      cout << "ERROR: Cannot get Notify Log factory" << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   DsLogAdmin::LogFullActionType logFullAction = DsLogAdmin::wrap;
   CORBA::ULongLong max_size = 0; // 0 means "infinite"
   CosNotification::QoSProperties qos= CosNotification::QoSProperties(0);
   CosNotification::QoSProperties adm= CosNotification::QoSProperties(0);
   DsLogAdmin::LogId logId = 0;
   DsLogAdmin::CapacityAlarmThresholdList_var thres = new DsLogAdmin::CapacityAlarmThresholdList ((CORBA::ULong)75);
   DsNotifyLogAdmin::NotifyLog_var notify_log;

   try
   {
      notify_log = factory->create (logFullAction, max_size, thres, qos, adm, logId);
   }
   catch(const DsLogAdmin::InvalidLogFullAction&)
   {
      cerr << "Notify Log creation failed with InvalidLogFullAction!" << endl;
      exit (INVALID_LOG_FULL_ACTION);
   }
   catch(const DsLogAdmin::InvalidThreshold&)
   {
      cerr << "Notify Log creation failed with InvalidThreshold!" << endl;
      exit (INVALID_THRESHOLD);
   }
   catch(const CosNotification::UnsupportedQoS&)
   {
      cerr <<"Unsupported QoS Setting..!" << endl;
      exit (UNSUPPORTED_QOS);
   }
   catch(const CosNotification::UnsupportedAdmin&)
   {
      cerr <<"Unsupported Admin Setting..!" << endl;
      exit (UNSUPPORTED_ADMIN);
   }


   if (CORBA::is_nil(notify_log.in ()))
   {
      cout << "ERROR: Notify Log is null." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   cout << "Created log with id :" << logId << endl;

   return notify_log;
}

void Notify::connect (DsNotifyLogAdmin::NotifyLog_var channel)
{
   CosNotifyChannelAdmin::SupplierAdmin_var admin = channel->default_supplier_admin ();
   CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::ANY_EVENT;
   CosNotifyChannelAdmin::ChannelID id;

   try
   {
      pc = admin->obtain_notification_push_consumer (type, id);
      proxy = CosNotifyChannelAdmin::ProxyPushConsumer::_narrow (pc.in ());

      proxy->connect_any_push_supplier
         (CosNotifyComm::PushSupplier::_narrow (getCORBARef ().in ()));
   }
   catch(const CosEventChannelAdmin::AlreadyConnected&)
   {
      cerr << "Already connected!"<< endl;
      exit (ALREADY_CONNECTED);
   }
   catch(const CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "Maximum number of suppliers exceeded.!" << endl;
      exit (ADMIN_LIMIT_EXCEEDED);
   }

   cout << "Connected supplier to log channel!" << endl;
}

void Notify::setFilter (DsNotifyLogAdmin::NotifyLog_var channel)
{
   CosNotifyFilter::FilterFactory_var filter_factory = channel->default_filter_factory ();
   CosNotifyFilter::Filter_var filter;
   char *grammar= "EXTENDED_TCL";

   try
   {
      filter = filter_factory->create_filter (grammar);
   }
   catch(const CosNotifyFilter::InvalidGrammar)
   {
      cerr << "-Invalid Grammar. Failed to create filter..!" << endl;
      exit (INVALID_GRAMMAR);
   }

   if (CORBA::is_nil (filter.in ()))
   {
      cerr << "Null Filter..." << endl;
      exit (NULL_OBJECT_REFERENCE);
   }

   try
   {
      CosNotification::EventTypeSeq evts;
      evts.length(0);
      CosNotifyFilter::ConstraintExp exp;
      exp.event_types= evts;

      char *constraint = "$._type_id == 'string'";

      exp.constraint_expr = ACE_OS::strdup (constraint);
      CosNotifyFilter::ConstraintExpSeq exps;
      exps.length(1);
      exps[0]= exp;
      filter->add_constraints(exps);
   }
   catch(const CosNotifyFilter::InvalidConstraint)
   {
      cerr << "Illegal constraint..." << endl;
      exit (INVALID_CONSTRAINT);
   }

   channel->set_filter (filter.in ());

   cout << "Set a filter on a Notify Log! " << endl;
}

void Notify::generate ()
{
   DsLogAdmin::Anys data (LOG_EVENTS);
   data.length (LOG_EVENTS);

   double d = 42.424242;
   int n = 123456789;
   char a = 'a';

   data[0] <<= ACE_OS::strdup ("Hello world");
   data[1] <<= d;
   data[2] <<= n;
   data[3] <<= a;

   for (int i=0; i < LOG_EVENTS; i++)
   {

      try
      {
         proxy->push (data[i]);
      }
      catch(const CosEventComm::Disconnected&)
      {
         cerr << "Proxy is disconnected..."<< endl;
         exit (DISCONNECTED);
      }
   }
   cout <<"Generated some log messages..." << endl;
}

void Notify::disconnect ()
{
   try
   {
      proxy->disconnect_push_consumer ();
   }
   catch(const CORBA::SystemException& ex)
   {
      cerr << "Caught " << ex << " while disconnecting." << endl;
      exit (SYSTEM_EXCEPTION);
   }
   catch(...)
   {
      cerr << "Unexpected exception" << endl;
      exit (SYSTEM_EXCEPTION);
   }

   cout << "Disconnected supplier from log channel." << endl;
}
