#include <ace/streams.h>
#include "Util.hh"

Util::Util(CORBA::ORB_ptr orb)
{
   _orb = orb;
}

void Util::buildNameGraph ()
{
   CosNaming::NamingContextExt_var rootCtx;
   CosNaming::NamingContext_var accMgrCtx;
   CosNaming::NamingContext_var acmeCtx;
   CosNaming::NamingContext_var accountsCtx;

   CORBA::Object_var obj;

   try
   {
      cout << "Building NameService Graph..." << endl;

      rootCtx = getRootNamingCtx ();
      CosNaming::Name_var name;

      try
      {
         name = rootCtx->to_name ("ACME_Bank");
         acmeCtx = rootCtx->bind_new_context (name);
      }
      catch(const CosNaming::NamingContext::AlreadyBound&)
      {
         acmeCtx = rootCtx->new_context ();
         rootCtx->rebind_context (name, acmeCtx.in ());
      }
      catch(const CORBA::Exception& ex)
      {
         ex._tao_print_exception ("buildNameGraph:Exception while binding ACME_Bank.");
         exit (-1);
      }

      try
      {
         name = rootCtx->to_name ("AccountManager");
         accMgrCtx = acmeCtx->bind_new_context (name);
      }
      catch(const CosNaming::NamingContext::AlreadyBound&)
      {
         accMgrCtx = rootCtx->new_context ();
         acmeCtx->rebind_context (name, accMgrCtx.in ());
      }
      catch(const CORBA::Exception& ex)
      {
         ex._tao_print_exception ("buildNameGraph:Exception while binding AccountManager.");
         exit (-1);
      }

      try
      {
         name = rootCtx->to_name ("Accounts");
         accountsCtx = acmeCtx->bind_new_context (name);
      }
      catch(const CosNaming::NamingContext::AlreadyBound&)
      {
         accountsCtx = rootCtx->new_context ();
         rootCtx->rebind_context (name, accountsCtx.in ());
      }
      catch(const CORBA::Exception& ex)
      {
         ex._tao_print_exception ("buildNameGraph:Exception while binding Accounts");
         exit (-1);
      }

      cout << "NameService Graph Build Succesfully!" << endl;
   }
   catch(const CosNaming::NamingContext::NotFound& nf)
   {
      nf._tao_print_exception ("buildNameGraph:NotFound");
      exit (-1);
   }
   catch(const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("buildNameGraph: exception.");
      exit (-1);
   }
}

CosNaming::NamingContextExt_var Util::getRootNamingCtx (void)
{
   CORBA::Object_var nsObj= NULL;

   try
   {
      nsObj = _orb->resolve_initial_references ("NameService");
   }
   catch (const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("getRootNamingCtx:Failed to resolve NameService");
      exit (-1);
   }

   CosNaming::NamingContextExt_var rootCtx;
   try
   {
      rootCtx = CosNaming::NamingContextExt::_narrow(nsObj.in ());
      return rootCtx;
   }
   catch (const CORBA::Exception& e)
   {
      e._tao_print_exception ("getRootNamingCtx:Failed to narrow NameService object.");
      exit (-2);
   }
}

Bank::AccountManager_var Util::getAccManagerObject (void)
{
   CosNaming::NamingContextExt_var rootCtx;
   CORBA::Object_var obj;

   try
   {
      rootCtx = getRootNamingCtx ();

      obj = rootCtx->resolve_str ("ACME_Bank/AccountManager/AccountManagerServer");

      if(CORBA::is_nil (obj.in ()))
      {
         cout << "AccountManager is nil!" << endl;
         return NULL;
      }

      Bank::AccountManager_var accManager = Bank::AccountManager::_narrow (obj.in ());

      return accManager;
   }
   catch(const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("getAccManagerObject:FAILED TO OBTAIN AccountManager.");
      return NULL;
   }
}

CosNaming::NamingContext_var Util::getAccManagerCtx (void)
{
   CosNaming::NamingContextExt_var rootCtx;
   CosNaming::NamingContext_var accMgrCtx;
   CORBA::Object_var obj;

   try
   {
      rootCtx = getRootNamingCtx ();
      obj = rootCtx->resolve_str ("ACME_Bank/AccountManager");

      accMgrCtx = CosNaming::NamingContextExt::_narrow (obj.in ());

      return accMgrCtx;
   }
   catch(const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("getAccManagerCtx:FAILED TO OBTAIN AccountManager Context.");
      return NULL;
   }
}

CosNaming::NamingContext_var Util::getAccountsCtx (void)
{
   CosNaming::NamingContextExt_var rootCtx;
   CosNaming::NamingContext_var accountsCtx;
   CORBA::Object_var obj;

   try
   {
      rootCtx = getRootNamingCtx ();
      obj = rootCtx->resolve_str ("ACME_Bank/Accounts");

      accountsCtx = CosNaming::NamingContextExt::_narrow (obj.in ());

      return accountsCtx;
   }
   catch(const CORBA::Exception&)
   {
      return NULL;
   }
}

void Util::bindAccountManager (CORBA::Object_ptr obj)
{
   CosNaming::NamingContext_var accMgrCtx;
   CosNaming::NamingContextExt_var rootCtx;
   CosNaming::Name_var name;

   try
   {
      rootCtx = getRootNamingCtx ();
      accMgrCtx = getAccManagerCtx ();

      name = rootCtx->to_name ("AccountManagerServer");

      // bind the AccountManagerServer into the root context from here
      accMgrCtx->bind (name, obj);
   }
   catch(const CosNaming::NamingContext::AlreadyBound&)
   {
      // catch already bound and rebind on server re-start
      try
      {
         cout << "bindAccountManager:WARNING: rebinding AccountManagerServer..." << endl;
         accMgrCtx->rebind (name, obj);

      }
      catch (const CORBA::Exception& e)
      {
         e._tao_print_exception ("bindAccountManager:Unexpected exception in rebinding....");
         exit (-1);
      }
   }
   catch(const CORBA::SystemException& sysEx)
   {
      sysEx._tao_print_exception ("bindAccountManager:SystemException");
      exit (-2);
   }
   catch(const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("bindAccountManager:Unexpected exception ...");
      exit (-3);
   }

   cout << "Successfully bound AccountManagerServer to NameService..." << endl;
}

PortableServer::POA_var Util::getPersistentRootPOA (void)
{
   try
   {
      CORBA::Object_var obj = _orb->resolve_initial_references ("RootPOA");

      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow (obj.in ());

      return rootPOA;
   }
   catch(const CORBA::Exception& ex)
   {
      ex._tao_print_exception ("getPersistentRootPOA:Unexpected exception during RIR for RootPOA...");
      exit (-1);
   }
}

Bank::Account_ptr Util::getAccount (const char* name)
{
   CosNaming::NamingContext_var accountsCtx;
   Bank::Account_ptr account;
   CosNaming::Name nameCmp;
   CORBA::Object_var obj;

   try
   {
      nameCmp.length (1);
      nameCmp[0].id = ACE_OS::strdup (name);

      accountsCtx = getAccountsCtx ();
      obj = accountsCtx->resolve (nameCmp);

      account = Bank::Account::_narrow (obj.in ());

      return account;
   }
   catch(const CORBA::Exception&)
   {
      return NULL;
   }
}

void Util::addAccount (const char* name, CORBA::Object_ptr obj)
{
   CosNaming::NamingContextExt_var rootCtx;
   CosNaming::Name_var _name;
   Bank::Account_ptr account = getAccount (name);
   CosNaming::NamingContext_var accountsCtx;

   if(CORBA::is_nil (account))
   {
      try
      {
         rootCtx = getRootNamingCtx ();
         _name = rootCtx->to_name (ACE_OS::strdup (name));

         accountsCtx = getAccountsCtx ();
         accountsCtx->bind (_name, obj);
      }
      catch(const CosNaming::NamingContext::AlreadyBound&)
      {
         //This is impossible but just in case we'll rebind.
         accountsCtx->rebind (_name, obj);
      }
   }
}
