/* Definitions, classes, and prototypes for an object that creates/references a single event
 * channel and provides the functions to create
 * Structured Push Supplier Proxy(s) (StructuredProxyPushConsumer) for sending mail messages.
 * The object also takes credentials of mail-users (user name) and provides functions to create
 * Structured Pull Consumer Proxy(s) (StructuredProxyPullSupplier). The consumer side proxy is
 * applied with a filter to distinguish it by user name. The object connects suppliers and can
 * push mail-events to the channel, and connects consumers and can pull mail-events from the
 * channel.
 */
#include <ace/OS.h>
#include <tao/corba.h>

#include <iostream.h>

#include "PushSupplier.hh"
#include "PullConsumer.hh"

#include <CosNotifyCommS.h>
#include <CosNotifyChannelAdminS.h>
#include <CosNotificationS.h>

#include <tao/corba.h>
#include <ace/OS.h>

#include "ChannelManager.hh"

/*
 * @func Constructs a ChannelManager object that creates an event channel
 * @parm | obj | a generic CORBA object
 * @parm | iChannelId | the channel id
 * @parm | bCreate | default true to always create channel
 * @rdesc cout and return on error
 */
ChannelManager::ChannelManager( CORBA::Object_var obj, int iChannelId, CORBA::Boolean bCreate)
   : m_channel( NULL), m_proxyPullSupplier( NULL), m_proxyPushConsumer( NULL)
{

   assert( obj.in ());

   if ( CORBA::is_nil( obj.in () ))
   {
      cout << "ERROR: NULL CORBA::Object_var" << endl;
      return;
   }

// factory
   CosNotifyChannelAdmin::EventChannelFactory_var factory= CosNotifyChannelAdmin::EventChannelFactory::_narrow( obj.in ());

   if ( CORBA::is_nil( factory.in() ))
   {
      cout << "ERROR: Cannot get channel factory" << endl;
      return;
   }

// channel
   try
   {
      m_channel= factory->get_event_channel( iChannelId);
      cout << "INFO: Obtained channel with id: " << iChannelId << endl;
   }
   catch( CosNotifyChannelAdmin::ChannelNotFound&)
   {
//    only retrieve an existing channel
      if ( !bCreate)
      {
         cout << "ERROR: Channel not found with id " << iChannelId << " does not exist"<< endl;
         return;
      }
//    always make a channel
      CosNotification::QoSProperties qos= CosNotification::QoSProperties(0);
      CosNotification::AdminProperties adm= CosNotification::AdminProperties(0);
      CosNotifyChannelAdmin::ChannelID id;
      m_channel= factory->create_channel( qos, adm, id);

      cout << "INFO: Created channel with id " << id << endl;
   }

}

/*
 * @func destroy this object
 *
 */
ChannelManager::~ChannelManager() {}

/*
 * @func Creates a proxy pull supplier object from a default admin
 * object using the constructed channel
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::getStructuredProxyPullSupplier()
{
   assert( m_channel.in ());

   m_proxyPullSupplier= NULL;
// admin object
   CosNotifyChannelAdmin::ConsumerAdmin_var admin= m_channel->default_consumer_admin();
   cout <<"INFO: Obtained default consumer admin." << endl;

   CosNotifyChannelAdmin::ProxyID pid;
   CosNotifyChannelAdmin::ClientType type= CosNotifyChannelAdmin::STRUCTURED_EVENT;
   CosNotifyChannelAdmin::ProxySupplier_var ps= NULL;

// proxy supplier object
   try
   {
      ps= admin->obtain_notification_pull_supplier( type, pid);

      if ( ps.in () == NULL)
      {
      cout <<"ERROR: ps==null" << endl;
      return 0;
      }
   }
   catch( CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "ERROR: Administration limit exceeded." << endl;
      return 0;
   }
   catch (CORBA::NO_IMPLEMENT &noImpEx)
   {
      cout << "Pull model not implemented for OpenFusion Version 4" << endl;
      return 0;
   }

// narrow to m_proxyPullSupplier
   m_proxyPullSupplier= CosNotifyChannelAdmin::StructuredProxyPullSupplier::_narrow( ps.in() );

   if ( m_proxyPullSupplier.in () == NULL)
   {
      cout << "m_proxyPullSupplier==NULL" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func Creates a proxy push consumer object from a default admin
 * object using the constructed channel
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::getStructuredProxyPushConsumer()
{
   assert( m_channel.in() );

   m_proxyPushConsumer= NULL;

// default admin
   CosNotifyChannelAdmin::SupplierAdmin_var admin= m_channel->default_supplier_admin();
   //cout <<"INFO: Obtained default supplier admin." << endl;

   CosNotifyChannelAdmin::ProxyID pid;
   CosNotifyChannelAdmin::ClientType type= CosNotifyChannelAdmin::STRUCTURED_EVENT;
   CosNotifyChannelAdmin::ProxyConsumer_var pc= NULL;
// proxy consumer object
   try
   {
      pc= admin->obtain_notification_push_consumer( type, pid);
      if ( pc.in() == NULL)
      {
      cout <<"ERROR: pc==NULL" << endl;
      return 0;
      }
   }
   catch( CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "ERROR: Administration limit exceeded." << endl;
      return 0;
   }

// narrow to m_proxyPushConsumer
   m_proxyPushConsumer= CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow( pc.in() );

   if ( m_proxyPushConsumer.in () == NULL)
   {
      cout << "m_proxyPushConsumer==NULL" << endl;
      return 0;
   }

   return 1;
}

//deprecated
CORBA::Boolean ChannelManager::getStructuredProxyPullSupplierIOR( CORBA::ORB_var pOrb, char **sResult) const
{

   if ( CORBA::is_nil (m_proxyPullSupplier.in ()))
   {
      cout << "ERROR: NULL Proxy Pull Supplier" << endl;
      return 0;
   }
   try
   {
      *sResult= pOrb->object_to_string( m_proxyPullSupplier);
   }
   catch (...)
   {
      cout << "ERROR: Cannot stringify Proxy Pull Supplier" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func sets this object state for a given user name and proxy pull supplier
 * object using the constructed channel
 *
 * @parm | sName | the name of the user
 *
 * @parm | obj | the user proxy pull supplier
 *
 * @rdesc returns false on error
 */
void ChannelManager::setUser( char *sName, CORBA::Object_var obj)
{
   setUserName( sName);
}

/*
 * @func assigns a proxy pull supplier to this object
 * object using the constructed channel
 *
 * @parm | obj | the user proxy pull supplier
 *
 * @rdesc returns false on error
 */
void ChannelManager::setStructuredProxyPushConsumer( CORBA::Object_var obj)
{
   m_proxyPullSupplier= CosNotifyChannelAdmin::StructuredProxyPullSupplier::_narrow( obj.in () );
}

/*
 * @func applies a filter to the proxy pull supplier with the property name and
 * value corresponding to the user name
 *
 * @parm | sName | the name of the filter property
 *
 * @parm | sValue | the user name
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::applyFilterToProxyPullSupplier( const char *sName, const char *sValue) const
{
   if ( CORBA::is_nil (m_proxyPullSupplier.in ()))
   {
      cout << "ERROR: NULL m_proxyPullSupplier" << endl;
      return 0;
   }
// factory
   CosNotifyFilter::FilterFactory_var factory= m_channel->default_filter_factory();

   if ( CORBA::is_nil( factory.in () ))
   {
      cout << "ERROR: cannot get default filter factory" << endl;
      return 0;
   }

   char *grammar= "EXTENDED_TCL";

// filter
   CosNotifyFilter::Filter_var filter;
   try
   {
      filter= factory->create_filter( grammar);
      m_proxyPullSupplier->add_filter( filter.in () );
   }
   catch (...)
   {
      cout << "ERROR: creating filter" << endl;
      return 0;
   }

   char sConstraint[40];
   ACE_OS::strcpy( sConstraint, "$");
   ACE_OS::strcat( sConstraint, sName);
   ACE_OS::strcat( sConstraint, "=='");
   ACE_OS::strcat( sConstraint, sValue);
   ACE_OS::strcat( sConstraint, "'");

// apply constraint
   try
   {
      CosNotification::EventType type;
      type.domain_name= ACE_OS::strdup( "mail");
      type.type_name= ACE_OS::strdup( "mailbox");

      CosNotification::EventTypeSeq evts;
      evts.length(1);
      evts[0]= type;
      CosNotifyFilter::ConstraintExp exp;
      exp.event_types= evts;
      exp.constraint_expr= ACE_OS::strdup( sConstraint);
      CosNotifyFilter::ConstraintExpSeq exps;
      exps.length(1);
      exps[0]= exp;
      filter->add_constraints( exps);
   }

   catch (...)
   {
      cout << "ERROR: adding filter constraints" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func applies a filter to the proxy pull supplier corresponding to the
 * current object properties
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::applyFilterToProxyPullSupplier() const
{

   return applyFilterToProxyPullSupplier( CONSTRAINT, getUserName());
}

/*
 * @func sets the given user name to this object
 *
 * @parm | sName | the user name
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::setUserName( char *sName)
{

   if ( ACE_OS::strlen( sName)>USERNAMELENGTH)
   {
      cout << "ERROR: proxy name too long" << endl;
      return 0;
   }
   else
   {
      ACE_OS::strcpy( m_cUserName, sName);
   }

   return 1;
}

/*
 * @func returns the current user name
 *
 * @rdesc the user name
 */
char* ChannelManager::getUserName() const
{

   return (char *)m_cUserName;
}

/*
 * @func pushes a structured event representing a mail message to the
 * Notification Service
 *
 * @parm | sName | the user name used in the filterable body of the message -
 * forcing the destination
 *
 * @parm | sMessage | the message to be sent
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::pushEvent( char *sName, char *sMessage) const
{
// declare event
   CosNotification::StructuredEvent event;

// configure event properties
   CosNotification::EventType type;
   type.domain_name = ACE_OS::strdup( "mail");
   type.type_name = ACE_OS::strdup( "mailbox");

// decalare header body
   CosNotification::PropertySeq variable;
   CosNotification::PropertySeq filterable;

// populate filter constraint
   filterable.length(1);
   filterable[0].name= ACE_OS::strdup(CONSTRAINT);
   char m_sName[40];
   ACE_OS::strcpy (m_sName,sName);
   filterable[0].value <<= ACE_OS::strdup (m_sName);

// declare and init fixed header
   CosNotification::FixedEventHeader fixed;
   fixed.event_type = type;
   fixed.event_name = ACE_OS::strdup( "message");

// init message
   CORBA::Any data ;
   char sValue[250];

   ACE_OS::strcpy (sValue, sMessage);
   data <<= ACE_OS::strdup (sValue);

   CosNotification::EventHeader header;
   header.fixed_header = fixed;
   header.variable_header = variable;

   event.header= header;
   event.filterable_data = filterable;

// populate event ROB
   event.remainder_of_body = data;

// push the event
   m_proxyPushConsumer->push_structured_event (event);

   return 1;
}

/*
 * @func connects a Notification Service supplier to this proxy push consumer
 *
 * @parm | pSupplier | the supplier object
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::connectSupplier (PushSupplier *pSupplier) const
{
// connect to proxy with an implicit servant activation on the root POA
   try
   {
      m_proxyPushConsumer->connect_structured_push_supplier( pSupplier->_this());
      cout <<"INFO: Connected supplier to this proxy." << endl;
   }
   catch (CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }

   return 1;
}

/*
 * @func connects a Notification Service consumer to the this proxy pull supplier
 *
 * @parm | pConsumer | the consumer object
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::connectConsumer(PullConsumer *pConsumer) const
{
// connect to proxy with an implicit servant activation on the root POA
   try
   {
      m_proxyPullSupplier->connect_structured_pull_consumer( pConsumer->_this());
      cout <<"INFO: Connected consumer to this proxy." << endl;
   }
   catch (CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }


   return 1;
}

/*
 * @func disconnects consumer from this proxy pull supplier
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::disconnectConsumer() const
{
   try
   {
      m_proxyPullSupplier->disconnect_structured_pull_supplier();
      cout <<"INFO: Disonnected consumer from this proxy." << endl;
   }
   catch (CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }


   return 1;
}

/*
 * @func disconnects supplier from this proxy push consumer
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::disconnectSupplier() const
{
   try
   {
      m_proxyPushConsumer->disconnect_structured_push_consumer();
      cout <<"INFO: Disonnected supplier from this proxy." << endl;
   }
   catch (CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }

   return 1;
}

/*
 * @func pulls an event from the Notification Service
 *
 * @rdesc returns false on error
 */
CORBA::Boolean ChannelManager::receiveEvent() const
{

   CORBA::Boolean bResult= 0;

   try
   {
      CosNotification::StructuredEvent *event;
      event= m_proxyPullSupplier->try_pull_structured_event( bResult);
      if ( bResult)
      {
         const char *sMessage;
         if ( event->remainder_of_body >>= sMessage)
         {
            cout << sMessage << endl;
         }
         else
         {
            cout << "no string to receive" << endl;
         }
      }
      else
      {
         cout << "nothing to receive" << endl;
      }
   }
   catch (CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }

   return 1;
}
