#include <ace/OS.h>
#include <iostream.h>

#include <CosNotifyCommS.h>
#include <CosNotifyChannelAdminS.h>
#include <CosNotificationS.h>

#include "PushConsumer.hh"

#include <tao/corba.h>

#include <string.h>

#include "ChannelManager.hh"

/*
 * @func Constructs a ChannelManager object that creates an event channel
 *
 * @parm | obj | a generic CORBA object
 *
 * @parm | iChannelId | the channel id
 *
 * @parm | bCreate | default 1 to always create channel
 *
 * @rdesc cout and return on error
 */
ChannelManager::ChannelManager( CORBA::Object_ptr obj, int iChannelId, CORBA::Boolean bCreate)
{
   if ( CORBA::is_nil( obj))
   {
      cout << "ERROR: NULL CORBA::Object_var" << endl;
      return;
   }

// factory
   CosNotifyChannelAdmin::EventChannelFactory_var factory= CosNotifyChannelAdmin::EventChannelFactory::_narrow( obj);

   if ( CORBA::is_nil( factory.in ()))
   {
      cout << "ERROR: Cannot get channel factory" << endl;
      return;
   }

// channel
   CosNotifyChannelAdmin::ChannelID cid( iChannelId);
   try
   {
      m_channel= factory->get_event_channel( cid);
      cout << "INFO: Obtained channel with id: " << iChannelId << endl;
   }
   catch(CosNotifyChannelAdmin::ChannelNotFound&)
   {
//    only retrieve an existing channel
      if ( !bCreate)
      {
         cout << "ERROR: Channel not found with id " << iChannelId << " does not exist"<< endl;
         return;
      }
//    always make a channel
      //CosNotification::QoSProperties qos= CosNotification::QoSProperties(0);
      CosNotification::QoSProperties qos= CosNotification::QoSProperties( 2);

      qos.length( 2);
      qos[0].name= CORBA::string_dup( CosNotification::ConnectionReliability);
      qos[0].value <<= CosNotification::Persistent;
      qos[1].name= CORBA::string_dup( CosNotification::EventReliability);
      qos[1].value <<= CosNotification::Persistent;
      CosNotification::AdminProperties adm= CosNotification::AdminProperties(0);
      CosNotifyChannelAdmin::ChannelID id;
      m_channel= factory->create_channel( qos, adm, id);

      cout << "INFO: Created channel with id " << iChannelId << ": Persistent" << endl;
      //setPersistentQos();
   }
}

/*
 * @func destroy this object
 *
 */
ChannelManager::~ChannelManager() {}

/*
 * @func Creates a proxy push supplier object from a default admin
 * object using the constructed channel
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::getStructuredProxyPushSupplier()
{

   if ( !CORBA::is_nil( m_proxyPushSupplier.in ()))
      return 1;

   CosNotifyChannelAdmin::ConsumerAdmin_var admin;
   try
   {
      admin= m_channel->default_consumer_admin();
      cout << "INFO: Obtained default consumer admin." << endl;
   }
   catch( CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "ERROR: Administration limit exceeded." << endl;
      return 0;
   }

   CosNotifyChannelAdmin::ProxyID pid;
   CosNotifyChannelAdmin::ClientType type= CosNotifyChannelAdmin::STRUCTURED_EVENT;
   CosNotifyChannelAdmin::ProxySupplier_var ps;

// proxy supplier object
   try
   {
      ps= admin->obtain_notification_push_supplier( type, pid);
      if ( CORBA::is_nil( ps.in ()))
      {
      cout <<"ERROR: ps==null" << endl;
      return 0;
      }
   }
   catch (...)
   {
      cout << "ERROR: obtaining proxy supplier" << endl;
      return 0;
   }

// narrow to m_proxyPushSupplier
   m_proxyPushSupplier= CosNotifyChannelAdmin::StructuredProxyPushSupplier::_narrow( ps.in ());

   if ( CORBA::is_nil( m_proxyPushSupplier.in ()))
   {
      cout << "proxyPushSupplier==NULL" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func Creates a proxy push consumer object from a default admin
 * object using the constructed channel
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::getStructuredProxyPushConsumer()
{

   if ( !CORBA::is_nil( m_proxyPushConsumer.in ()))
     return 1;

   CosNotifyChannelAdmin::SupplierAdmin_var admin;
   try
   {
      admin= m_channel->default_supplier_admin();
      cout << "INFO: Obtained default supplier admin." << endl;
   }
   catch( CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "ERROR: Administration limit exceeded." << endl;
      return 0;
   }

   CosNotifyChannelAdmin::ProxyID pid;
   CosNotifyChannelAdmin::ClientType type= CosNotifyChannelAdmin::STRUCTURED_EVENT;
   CosNotifyChannelAdmin::ProxyConsumer_var pc;
// proxy consumer object
   try
   {
      pc= admin->obtain_notification_push_consumer( type, pid);
      if ( CORBA::is_nil( pc.in ()))
      {
      cout <<"ERROR: pc==NULL" << endl;
      return 0;
      }
   }
   catch( CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "ERROR: Administration limit exceeded." << endl;
      return 0;
   }

// narrow to m_proxyPushConsumer
   m_proxyPushConsumer= CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow( pc.in ());

   if ( CORBA::is_nil( m_proxyPushConsumer.in ()))
   {
      cout << "ERROR: creating proxy push consumer" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func Creates a proxy push supplier object from a default admin
 * object using the constructed channel
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::getStructuredProxyPushSupplierFromId()
{
   CosNotifyChannelAdmin::ConsumerAdmin_var admin;
   try
   {
      admin= m_channel->default_consumer_admin();
      cout << "INFO: Obtained default consumer admin" << endl;
   }
   catch( CosNotifyChannelAdmin::AdminLimitExceeded&)
   {
      cerr << "ERROR: Administration limit exceeded." << endl;
      return 0;
   }

   //CORBA::ULong lId= 0;
   CosNotifyChannelAdmin::ProxyID pid= 0;
   CosNotifyChannelAdmin::ClientType type= CosNotifyChannelAdmin::STRUCTURED_EVENT;
   CosNotifyChannelAdmin::ProxySupplier_var ps;

   try
   {
      ps= admin->get_proxy_supplier( pid);
      //cout << "INFO: got admin from id" << endl;
      if ( CORBA::is_nil( ps.in ()))
      //if ( ps==NULL)
      {
         cout <<"ERROR: ps==null" << endl;
         return 0;
      }
   }
   catch ( CosNotifyChannelAdmin::ProxyNotFound& e)
   {
      ACE_PRINT_EXCEPTION (e, "getStructuredProxyPushSupplierFromId");
      return 0;
   }
   catch (...)
   {
      cerr << "ERROR: getting proxy id: " << pid << endl;
      return 0;
   }


// narrow to m_proxyPushSupplier
   m_proxyPushSupplier= CosNotifyChannelAdmin::StructuredProxyPushSupplier::_narrow( ps.in ());
   //cout << "INFO: narrowed" << endl;

   if ( CORBA::is_nil( m_proxyPushSupplier.in ()))
   //if ( m_proxyPushSupplier==NULL)
   {
      cout << "proxyPushSupplier==NULL" << endl;
      return 0;
   }

   return 1;
}
/*
 * @func connect the supplier to the channel
 *
 * @parm | pSupplier | the given StructuredPushSupplier
 *
 * @rdesc cout and return on error
 */
CORBA::Boolean ChannelManager::connectSupplier( CosNotifyComm::StructuredPushSupplier_ptr pSupplier, const CORBA::Boolean bPublisher)
{
   if ( CORBA::is_nil( m_proxyPushConsumer.in ()))
   {
      if ( !getStructuredProxyPushConsumer())
         return 0;
   }

   if ( bPublisher)
      if ( !getPublisher())
         return 0;

   try
   {
      m_proxyPushConsumer->connect_structured_push_supplier( pSupplier);
      cout <<"INFO: Connected supplier to this proxy." << endl;
   }
   catch (CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }

   return 1;
}

/*
 * @func connects a Notification Service consumer to the this proxy pull supplier
 *
 * @parm | pConsumer | the consumer object
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::connectConsumer( CosNotifyComm::StructuredPushConsumer_ptr pConsumer, int& iOpt)
{
   if ( iOpt==1)
   {
      if ( !getStructuredProxyPushSupplier())
         return 0;
   }
   else
   {
      if ( !getStructuredProxyPushSupplierFromId())
         return 0;
   }

   try
   {
      m_proxyPushSupplier->connect_structured_push_consumer( pConsumer);
      cout <<"INFO: Connected consumer to this proxy." << endl;
   }
   catch( CosEventChannelAdmin::AlreadyConnected& e)
   {
      ACE_PRINT_EXCEPTION (e, "connectConsumer");
      if ( iOpt==1)
      {
         cout << endl;
         return 0;
      }
      else
      {
         cout << "- ignored" << endl;
      }
   }
   catch (const CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }
   catch (...)
   {
      cout << "ERROR: problem connecting consumer to this proxy" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func Sets qos properties for persistent events to constructed channel
 *
 * @rdesc void
 */
void ChannelManager::setPersistentQos() const
{
   assert( CORBA::is_nil (m_channel.in ()));
   CosNotification::QoSProperties qos= CosNotification::QoSProperties( 2);

   qos.length( 2);
   qos[0].name= CORBA::string_dup( CosNotification::ConnectionReliability);
   qos[0].value <<= CosNotification::Persistent;
   qos[1].name= CORBA::string_dup( CosNotification::EventReliability);
   qos[1].value <<= CosNotification::Persistent;

   try
   {
      m_channel->set_qos( qos);
      cout << "INFO: Set persistent QoS" << endl;
   }
   catch (CosNotification::UnsupportedQoS&)
   {
      cerr << "ERROR: UnsupportedQoS" << endl;
   }
   catch (...)
   {
      cerr << "ERROR: Setting QoS" << endl;
   }
}

void ChannelManager::setStructuredProxyPushSupplier( CORBA::Object_ptr pProxy)
{
   m_proxyPushSupplier= CosNotifyChannelAdmin::StructuredProxyPushSupplier::_narrow( pProxy);
}

CosNotifyChannelAdmin::StructuredProxyPushSupplier_var ChannelManager::returnStructuredProxyPushSupplier()
{
   return m_proxyPushSupplier;
}

/*
 * @func Creates a NotifyPublish publisher to advertise the events to be published
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::getPublisher()
{
   try
   {
      m_publisher= CosNotifyComm::NotifyPublish::_narrow( m_proxyPushConsumer.in ());
      cout << "INFO: created publisher" << endl;
   }
   catch (...)
   {
      cerr << "ERROR: cannot narrow to publisher" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func Creates a NotifySubscribe subscriber to advertise the events to be subscribed to
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::getSubscriber()
{
   try
   {
      m_subscriber= CosNotifyComm::NotifySubscribe::_narrow( m_proxyPushSupplier.in ());
   }
   catch (...)
   {
      cout << "ERROR: cannot narrow to subscriber" << endl;
      return 0;
   }

   return 1;
}

/*
 * @func suspends the consumer connection
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::suspend() const
{
   if ( CORBA::is_nil( m_proxyPushSupplier.in ()))
      return 0;

   try
   {
      m_proxyPushSupplier->suspend_connection();
      cout <<"INFO: Suspended consumer connection" << endl;
   }
   catch ( const CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }

   return 1;
}

/*
 * @func resumes the consumer connection
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean ChannelManager::resume() const
{
   if ( CORBA::is_nil( m_proxyPushSupplier.in ()))
      return 0;

   try
   {
      m_proxyPushSupplier->resume_connection();
      cout <<"INFO: Resumed consumer connection" << endl;
   }
   catch ( const CORBA::Exception& e)
   {
      cout << e << endl;
      return 0;
   }

   return 1;
}
