#include <ace/OS.h>
#include <iostream.h>

#include <CosNotifyCommS.h>
#include <CosNotifyChannelAdminS.h>
#include <CosNotificationS.h>

#include <tao/corba.h>


#include "ChannelManager.hh"
#include "NewsChannelManager.hh"

/*
 * @func Constructs a NewsChannelManager object that creates an event channel
 *
 * @parm | obj | the given Notification Service Object
 *
 * @parm | iChannelId | the given channel id
 *
 * @parm | bCreate | default true to always create channel
 *
 * @rdesc cout and return on error
 */
NewsChannelManager::NewsChannelManager( CORBA::Object_ptr obj, int iChannelId, CORBA::Boolean bCreate) :
                     ChannelManager( obj, iChannelId, bCreate)
{
}

/*
 * @func destructs a NewsChannelManager object
 */
NewsChannelManager::~NewsChannelManager(){}

/*
 * @func inform channel of new event type
 *
 * @parm | iType | the given event-type enumeration
 */
void NewsChannelManager::addOffer( const int iType)
{
   if ( CORBA::is_nil( m_publisher.in ()))
   {
      cout << "ERROR: no publisher object" << endl;
      return;
   }
   CosNotification::EventTypeSeq eventsAdded;
   eventsAdded.length( 1);
   CosNotification::EventTypeSeq eventsRemoved;
   eventsRemoved.length( 0);
   CosNotification::EventType type;

   try
   {
      switch( iType)
      {
         case 0:
            type.domain_name= ACE_OS::strdup( "weather");
            type.type_name= ACE_OS::strdup( "news agency");
            break;
         case 1:
            type.domain_name= ACE_OS::strdup( "stock");
            type.type_name= ACE_OS::strdup( "news agency");
            break;
         default:
            cout << "WARNING: weather or stock news only" << endl;
            return;
      };
      eventsAdded[0]= type;
   }
   catch (...)
   {
      cout << "ERROR: cannot change offer" << endl;
   }

}

/*
 * @func inform channel of event type top remove
 *
 * @parm | iType | the given event-type enumeration
 */
void NewsChannelManager::removeOffer( const int iType)
{
   if ( CORBA::is_nil( m_publisher.in ()))
   {
      cout << "ERROR: no publisher object" << endl;
      return;
   }
   CosNotification::EventTypeSeq eventsAdded;
   eventsAdded.length( 0);
   CosNotification::EventTypeSeq eventsRemoved;
   eventsRemoved.length( 1);
   CosNotification::EventType type;

   try
   {
      switch( iType)
      {
         case 0:
            type.domain_name= ACE_OS::strdup( "weather");
            type.type_name= ACE_OS::strdup( "news agency");
            break;
         case 1:
            type.domain_name= ACE_OS::strdup( "stock");
            type.type_name= ACE_OS::strdup( "news agency");
            break;
         default:
            cout << "WARNING: weather or stock news only" << endl;
            return;
      };
      eventsRemoved[0]= type;
   }
   catch (...)
   {
      cout << "ERROR: cannot change offer" << endl;
   }

}

/*
 * @func pushes a structured event representing a mail message to the
 * Notification Service
 *
 * @parm | sMessage | the message to be sent
 *
 * @parm | iPriority | the priority filter value
 *
 * @parm | iType | the type of message to be sent
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean NewsChannelManager::pushEvent( const char *sMessage, const int iPriority, const int iType) const
{

// declare event
   CosNotification::StructuredEvent event;

// configure event properties
   CosNotification::EventType type;

   switch ( iType)
   {
      case 0:
         type.domain_name= ACE_OS::strdup( "weather");
         type.type_name= ACE_OS::strdup( "news agency");
         break;
      case 1:
         type.domain_name= ACE_OS::strdup( "stock");
         type.type_name= ACE_OS::strdup( "news agency");
         break;
      default:
         cout << "ERROR: event type not recognised" << endl;
         return 0;
   };

// decalare header body
   CosNotification::PropertySeq variable;
   CosNotification::PropertySeq filterable;
// populate filter constraint
   filterable.length( 1);
   //variable.length( 0);
   filterable[0].name= ACE_OS::strdup( CONSTRAINT);
   CORBA::UShort usPriority= iPriority;
   filterable[0].value <<= usPriority;


   CosNotification::FixedEventHeader fixed;
   fixed.event_type= type;
   fixed.event_name= ACE_OS::strdup( "message");

// init message
   CORBA::Any data;
   data <<= sMessage;
   CosNotification::EventHeader header;
   header.fixed_header= fixed;
   header.variable_header= variable;

   event.header= header;
   event.filterable_data= filterable;

// populate event ROB
   event.remainder_of_body= data;

// push the event
   try
   {
      m_proxyPushConsumer->push_structured_event( event);
      cout << "INFO: Message sent: " << sMessage << ": " <<  CONSTRAINT << ": " << iPriority << ": domain- " << type.domain_name << endl;
   }
   catch (...)
   {
      cout << "ERROR: sending message" << endl;
   }


   return 1;
}

/*
 * @func applies a filter to the proxy push supplier with the property name and
 * value corresponding to the required event type and priority
 *
 * @parm | sName | the name of the filter property
 *
 * @parm | sValue | the user name
 *
 * @rdesc returns 0 on error
 */
CORBA::Boolean NewsChannelManager::applyFilter( const int iType, const int iPriority) const
{

   if ( CORBA::is_nil( m_proxyPushSupplier.in ()))
   {
      cout << "ERROR: no proxy push supplier" << endl;
      return 0;
   }
// factory
   CosNotifyFilter::FilterFactory_var factory= m_channel->default_filter_factory();

   if ( CORBA::is_nil( factory.in ()))
   {
      cout << "ERROR: cannot get default filter factory" << endl;
      return 0;
   }

   char *grammar= "EXTENDED_TCL";

// filter
   CosNotifyFilter::Filter_var filter;

   try
   {
      filter= factory->create_filter( grammar);
      int id= m_proxyPushSupplier->add_filter( filter.in ());
   }
   catch (...)
   {
      cout << "ERROR: adding filter" << endl;
      return 0;
   }


   char sConstraint[40];
   ACE_OS::strcpy( sConstraint, "$");
   ACE_OS::strcat( sConstraint, CONSTRAINT);
   ACE_OS::strcat( sConstraint, " > ");
   char sValue[4];
   sprintf( sValue, "%d", iPriority);
   ACE_OS::strcat( sConstraint, sValue);

   cout << "INFO: constraint: " << sConstraint << endl;

// apply constraint
   try
   {
      CosNotification::EventType type;
      switch( iType)
      {
         case 0:
            type.domain_name= ACE_OS::strdup( "weather");
            type.type_name= ACE_OS::strdup( "news agency");
            break;
         case 1:
            type.domain_name= ACE_OS::strdup( "stock");
            type.type_name= ACE_OS::strdup( "news agency");
            break;
         default:
            cout << "WARNING: weather or stock news only" << endl;
            return 0;
      };

      CosNotification::EventTypeSeq evts;
      evts.length( 1);
      evts[0]= type;
      CosNotifyFilter::ConstraintExp exp;
      exp.event_types= evts;
      exp.constraint_expr= ACE_OS::strdup( sConstraint);
      CosNotifyFilter::ConstraintExpSeq exps;
      exps.length( 1);
      exps[0]= exp;
      filter->add_constraints( exps);
   }
   catch (...)
   {
      cout << "ERROR: creating filter" << endl;
      return 0;
   }


   return 1;
}
