#include <ace/OS.h>
#include <iostream.h>
#include <tao/corba.h>

#include "TradingServer.hh"
#include "tao/Typecode.h"


/* This program is a CORBA server that delegates to a VideoServiceProvider object
 * (singleton) that manages VideoSupplier objects using the Trading Service. The program
 * registers a Service Type describing a VideoSupplier offer. Each
 * VideoSupplier object exports it's offer to the Trader
 */
int main ( int argc, char **argv)
{
   try
   {
   // Utilities
      Accessor utils;
   
   // initialise orb
      CORBA::ORB_var pOrb= NULL;
      try
      {
         pOrb= utils.getORB(argc, argv);
         cout << "INFO: initialised orb" << endl;
      }
      catch(CORBA::Exception& e)
      {
         cout << "ERROR: Cannot initialise orb: " << e << endl;
         return 1;
      }
   
   // get reference to trading service
      CORBA::Object_var pObj= NULL;
   
      try
      {
         pObj = pOrb->resolve_initial_references ("TradingService");
      }
      catch (...)
      {
         cerr << "Failed to resolve TradingService" << endl;
         return -1;
      }
   
   
      assert( pObj.in ());
   
      if ( CORBA::is_nil( pObj.in ()))
      {
         return 1;
      }
   
      CosTrading::Lookup_var lookup;
      try
      {
         lookup= CosTrading::Lookup::_narrow( pObj.in ());
      }
      catch (...)
      {
         cout << "ERROR: narrowing to lookup" << endl;
         return 1;
      }
   
      CORBA::Object_var obj= lookup->type_repos();
      CosTradingRepos::ServiceTypeRepository_var repos;
      try
      {
         repos= CosTradingRepos::ServiceTypeRepository::_narrow( obj.in ());
      }
      catch (...)
      {
         cout << "ERROR: narrowing ServiceTypeRepository" << endl;
         return 1;
      }
   
      VideoSupplier::m_pServiceProvider = new VideoServiceProvider (pOrb);
      assert( VideoSupplier::m_pServiceProvider);
   
   // register service type
      CosTradingRepos::ServiceTypeRepository::PropStructSeq properties;
   
      properties.length( 3);
   
      properties[0].name= ACE_OS::strdup( "type");
      properties[0].value_type= CORBA::_tc_string;
      properties[0].mode= CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY_READONLY;
   
      properties[1].name= ACE_OS::strdup( "title");
      properties[1].value_type= CORBA::_tc_string;
      properties[1].mode= CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY_READONLY;
   
      properties[2].name= ACE_OS::strdup( "cost");
      properties[2].value_type= CORBA::_tc_float;
      properties[2].mode= CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
   
      CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq defaultSupers;
   
      try
      {
         repos->add_type( "IDL:video_supplier:1.0", "IDL:video_supplier_if:1.0", properties, defaultSupers);
      }
      catch( CosTradingRepos::ServiceTypeRepository::ServiceTypeExists& ste)
      {
         ACE_PRINT_EXCEPTION(ste, "main");
         //return 1;
      }
      catch( CosTradingRepos::ServiceTypeRepository::InterfaceTypeMismatch& e)
      {
         ACE_PRINT_EXCEPTION(e, "main");
         return 1;
      }
      catch( CosTrading::DuplicatePropertyName& e)
      {
         ACE_PRINT_EXCEPTION(e, "main");
         return 1;
      }
      catch( CosTradingRepos::ServiceTypeRepository::ValueTypeRedefinition& e)
      {
         ACE_PRINT_EXCEPTION(e, "main");
         return 1;
      }
      catch( CosTrading::UnknownServiceType& e)
      {
         ACE_PRINT_EXCEPTION(e, "main");
         return 1;
      }
      catch (...)
      {
         cout << "ERROR: adding type" << endl;
         return 1;
      }
   
   
   
      /////////////////////////////////////////////////////
   // create a few arbitary categories
      char *cats[]= {   "Films",          //0
                        "Sporting Events",//1
                        "News Broadcasts",//2
                        "Soap Operas"     //3
                        };
   
   // create a few arbitary videos
      char *vids[]= {   "Bridge On The River Kwai",//0
                        "Mutiny On The Bounty",    //1
                        "Lawrence Of Arabia",      //2
                        "Star Wars",               //3
                        "Great Expectations",      //4
                        "FA Cup Final",            //5
                        "BBC",                     //6
                        "CNN",                     //7
                        "Coronation Street",       //8
                        "Neighbours"               //9
                     };
   
      char *video1= "Videos' R us";
      char *video2= "Videos' for Sale";
      char *video3= "Videos' on Demand";
   
   // create a few arbitary video suppliers these export their offers
      TradingVideoSupplier videosRUs( pOrb, video1);
      videosRUs.addCategory( cats[0]);
      videosRUs.addCategory( cats[1]);
      videosRUs.addVideo( cats[0], vids[0]);
      videosRUs.addVideo( cats[0], vids[1]);
      videosRUs.addVideo( cats[1], vids[5]);
      videosRUs.exportOffer( pOrb, lookup.in ());
   
      TradingVideoSupplier videos4Sale( pOrb, video2);
      videos4Sale.addCategory( cats[0]);
      videos4Sale.addCategory( cats[2]);
      videos4Sale.addVideo( cats[0], vids[2]);
      videos4Sale.addVideo( cats[0], vids[3]);
      videos4Sale.addVideo( cats[0], vids[4]);
      videos4Sale.addVideo( cats[2], vids[6]);
      videos4Sale.addVideo( cats[2], vids[7]);
      videos4Sale.exportOffer( pOrb, lookup.in ());
   
      TradingVideoSupplier videosOnDemand( pOrb, video3);
      videosOnDemand.addCategory( cats[2]);
      videosOnDemand.addCategory( cats[3]);
      videosOnDemand.addVideo( cats[2], vids[6]);
      videosOnDemand.addVideo( cats[3], vids[8]);
      videosOnDemand.addVideo( cats[3], vids[9]);
      videosOnDemand.exportOffer( pOrb, lookup.in ());
   
      VideoSupplier::m_pServiceProvider->activateManager();
   
   // accept incoming requests on this server
      cout << "Ready..." << endl;
      pOrb->run();
   
   // clean up global pointer
      delete VideoSupplier::m_pServiceProvider;
   
      return 0;
   }
   catch(CORBA::Exception& sysEx)
   {
      cerr << sysEx << endl;
      return 1;
   }
}
